#title: "lncRNA co-expression network"

# First, you have to install the RTN package.

if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("RTN")
library(RTN)

# Import gene expression matrix to R enviroment. Here it`s important that gene are organiezed
# in rownames and patients samples in column names

Expression_Table <- read.table("RNA_BRCA.txt", header = T)

# You can select specific lncRNAs to compute the co-expression network. For example, genes with
# higher variance. In this example, the user can select 50% most variable lncRNAs

Expression_Table$variance = apply(Expression_Table, 1, var)
Expression_Table2 = Expression_Table[Expression_Table$variance >= quantile(Expression_Table$variance, c(.50)), ] #50% most variable genes
Expression_Table2$variance <- NULL

#The next step is to compute the co-expression network for some selected lncRNAs.


#First the user needs to create a vector with lncRNA annotation. In this example, we are using LINC00504 as described in the article
tfs <- "ENSG00000248360"

#To compute the co-expression networks we used RTN package. It`s complete vignette is deposited at Bioconductor.
rtni_lncs <- tni.constructor(expData = Expression_Table2, regulatoryElements = tfs)

# User can change pValueCutoff.
rtni_lncs <- tni.permutation(rtni_lncs, pValueCutoff = 10^-8, 
                             estimator = "spearman")

#We add at this point the option to save the permutation results since it`s demands high processing by the computer.
save(rtni_lncs, file = "rtni_lncs_perm10-8.RData")
rtni_lncs <- tni.bootstrap(rtni_lncs, estimator = "spearman")
rtni_lncs <- tni.dpi.filter(rtni_lncs)

#Save the final object. This one is used to futher analysis.
save(rtni_lncs, file = "rtni_lncs10-8.RData")

#Get the regulons data
regs <- tni.get(rtni_lncs, what="regulons")

#With this object the user can explore specific regulons as:
regs[["ENSG00000248360"]]

#Or explore all of them based on a parameter. In this article, we decided to focus on difference
# in breast cancer molecular subtypes. To do this, we applied a statistical method called Kruskal-Wallis followed by Dunn`s test
# To do this, we also need to import a clinical table to classify the samples according to its molecular subtype.

reg.names <- names(regs)

Results_table <- NULL

for(reg in reg.names)
{
  dES <- rtns@EScores$dif
  dES_vector <- dES[,reg]
  kw.table <- cbind(dES_vector, clinical$PAM50.mRNA)
  colnames(kw.table) <- c("dES", "Subtype")
  
  #-- p valor kw
  p.kw <- kruskal.test(dES ~ Subtype, data = kw.table)$p.value
  
  #-- p valor dunn
  result <- dunn.test::dunn.test(kw.table[,"dES"], kw.table[,"Subtype"], 
                                 method = "bonferroni", table = FALSE, kw = FALSE)
  p.dunn <- result$P.adjusted[c(1,2,4)]
  
  #--mean dES by subtype
  mean_dES <- function(kw.table, subtype.value)
  {
    
    subtype <- which(kw.table[,"Subtype"] == subtype.value)
    return(mean(dES_vector[subtype])) 
  }
  dES_basal <- mean_dES(kw.table, 1)
  dES_her <- mean_dES(kw.table, 2)
  dES_lumA <- mean_dES(kw.table, 3)
  dES_lumB <- mean_dES(kw.table, 4)
  #--- Get regulon size
  reg.size <- length(regs[[reg]])
  #--- Get genes positive correlated
  pos.cor <- sum(tnet[,reg] > 0)
  #-- Get genes negative correlated
  neg.cor <- sum(tnet[,reg] < 0)
  
Results <- c(reg, p.kw, p.dunn, dES_basal, dES_her, dES_lumA, dES_lumB, reg.size,pos.cor,neg.cor)
  
Final_results <- rbind(Final_results,Results)
  
}

rownames(Final_results)<- rep(NA, length(rownames(Final_results)))
colnames(Final_results)<- c("regulon_name", "p.valor kw", "p.dunnBasxHER", "p.dunnBasxLA","p.dunnBasxLB", "dES_basal", "dES_her","dES_lumA","dES_lumB","reg size","pos.cor","neg.cor")

